/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Ken VanDine <ken@vandine.org>
 */

using Gwibber;

namespace Gwibber.Test
{
  public class UtilsSuite
  {
    public UtilsSuite ()
    {
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/time_string_seconds",
                               UtilsSuite.test_time_string_seconds);
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/time_string_minute",
                               UtilsSuite.test_time_string_minute);
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/time_string_minutes",
                               UtilsSuite.test_time_string_minutes);
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/time_string_hour",
                               UtilsSuite.test_time_string_hour);
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/time_string_hours",
                               UtilsSuite.test_time_string_hours);
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/time_string_day",
                               UtilsSuite.test_time_string_day);
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/time_string_days",
                               UtilsSuite.test_time_string_days);
      GLib.Test.add_data_func ("/Unit/Gwibber/Utils/avatar_path",
                               UtilsSuite.test_avatar_path);
    }

    internal static void test_time_string_seconds ()
    {
      var utils = new Gwibber.Utils ();
      var t = ((int)Time.gm (time_t ()).mktime() - 3600) - 29;
      var ts = utils.generate_time_string (t);
      assert (ts == "a few seconds ago");
    }

    internal static void test_time_string_minute ()
    {
      var utils = new Gwibber.Utils ();
      var t = ((int)Time.gm (time_t ()).mktime() - 3600) - 60;
      var ts = utils.generate_time_string (t);
      assert (ts == "1 minute ago");
    }
    internal static void test_time_string_minutes ()
    {
      var utils = new Gwibber.Utils ();
      var t = ((int)Time.gm (time_t ()).mktime() - 3600) - 121;
      var ts = utils.generate_time_string (t);
      assert (ts.contains ("minutes ago"));
    }
    internal static void test_time_string_hour ()
    {
      var utils = new Gwibber.Utils ();
      var t = ((int)Time.gm (time_t ()).mktime() - 3600) - 3601;
      var ts = utils.generate_time_string (t);
      assert (ts == "1 hour ago");
    }
    internal static void test_time_string_hours ()
    {
      var utils = new Gwibber.Utils ();
      var t = ((int)Time.gm (time_t ()).mktime() - 3600) - 7201;
      var ts = utils.generate_time_string (t);
      assert (ts.contains ("hours ago"));
    }
    internal static void test_time_string_day ()
    {
      var utils = new Gwibber.Utils ();
      var t = ((int)Time.gm (time_t ()).mktime() - 3600) - 86400;
      var ts = utils.generate_time_string (t);
      assert (ts == "1 day ago");
    }
    internal static void test_time_string_days ()
    {
      var utils = new Gwibber.Utils ();
      var t = ((int)Time.gm (time_t ()).mktime() - 3600) - 172800;
      var ts = utils.generate_time_string (t);
      assert (ts.contains ("days ago"));
    }
    internal static void test_avatar_path ()
    {
      var utils = new Gwibber.Utils ();
      string url = "http://test.com/avatar";
      string avatar_path = utils.avatar_path (url);
      assert (avatar_path == Path.build_path (Path.DIR_SEPARATOR_S, Environment.get_user_cache_dir(), "gwibber/avatars", url.replace ("/","")));
    }
  }
}
