using Gwibber;

namespace Gwibber.Test
{
  [DBus (name = "org.freedesktop.DBus")]
  private interface FreeDesktop.DBusObject: Object {
    private abstract signal void name_owner_changed (string name,
                                                    string old_owner,
                                                    string new_owner);
    public abstract async string[] list_names () throws DBusError;
  }

  private class Utils : Object
  {
    private static DBusConnection connection;
    private string interface_name;

    public signal void available (bool up);

    construct
    {
      try {
        connection = Bus.get_sync (BusType.SESSION);
        connection.signal_subscribe ("org.freedesktop.DBus",
            "org.freedesktop.DBus", "NameOwnerChanged",
            "/org/freedesktop/DBus",
             null,
             0,
             on_name_owner_changed);

        check_interface.begin ();
      } catch (GLib.IOError e) {
        warning (e.message);
      }
    }

    private async void check_interface ()
    {
      FreeDesktop.DBusObject dbus_obj = Bus.get_proxy_sync (BusType.SESSION,
                                         "org.freedesktop.DBus",
                                         "/org/freedesktop/DBus",
                                         DBusProxyFlags.DO_NOT_LOAD_PROPERTIES);

      var services = yield dbus_obj.list_names ();
      foreach (var service in services) {
        if (service == interface_name)
          available (true);
      }
    }

    private void on_name_owner_changed (DBusConnection connection, string sender_name, string object_path, string iface_name, string signal_name, Variant parameters)
    {
      unowned string name, old_owner, new_owner;
      parameters.get ("(&s&s&s)", out name, out old_owner, out new_owner);

      if (name == (string)interface_name)
      {
        if (new_owner != "" && old_owner == "")
        {
          message ("%s is available", name);
          available (true);
        }
        if (old_owner != "" && new_owner == "")
        {
          available (false);
          message ("%s is no longer available", name);
        }
      }
    }

    public void setup (string name) throws GLib.IOError
    {
      interface_name = name;
    }
  }
}
